@@ -2,7 +2,7 @@
 
 # doublex
 #
-# Copyright © 2012 David Villa Alises
+# Copyright © 2012,2013 David Villa Alises
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 import itertools
 import threading
+import collections
 
 import hamcrest
 from hamcrest.core.base_matcher import BaseMatcher
@@ -28,10 +29,10 @@ from hamcrest.core.base_matcher import BaseMatcher
 try:
     from functools import total_ordering
 except ImportError:
-    from py27_backports import total_ordering
+    from .py27_backports import total_ordering
 
 
-import safeunicode
+from .safeunicode import get_string
 
 
 class WrongApiUsage(Exception):
@@ -93,23 +94,23 @@ class Method(Observable):
         super(Method, self).__init__()
         self.double = double
         self.name = name
-        self.event = threading.Event()
+        self._event = threading.Event()
 
     def __call__(self, *args, **kargs):
         if not self.double._setting_up:
             self.notify(*args, **kargs)
 
-        self.event.set()
+        self._event.set()
+
         invocation = self.create_invocation(args, kargs)
         return self.double._manage_invocation(invocation)
 
     def create_invocation(self, args, kargs):
-        return Invocation.from_args(
-            self.double, self.name, args, kargs)
+        return Invocation.from_args(self.double, self.name, args, kargs)
 
     def _was_called(self, context, times):
         invocation = Invocation(self.double, self.name, context)
-        return self.double._was_called(invocation, times)
+        return self.double._received_invocation(invocation, times)
 
     def describe_to(self, description):
         pass
@@ -165,7 +166,7 @@ class Invocation(object):
         return Invocation(double, name, InvocationContext(*args, **kargs))
 
     def delegates(self, delegate):
-        if callable(delegate):
+        if isinstance(delegate, collections.Callable):
             self.context.delegate = delegate
             return
 
@@ -273,7 +274,8 @@ class InvocationContext(object):
         return self.matches(other)
 
     def __lt__(self, other):
-        return (self.args, self.kargs) < (other.args, other.kargs)
+        return (self.args, sorted(self.kargs.items())) < \
+            (other.args, sorted(other.kargs.items()))
 
     def __str__(self):
         return str(InvocationFormatter(self))
@@ -294,25 +296,29 @@ class InvocationFormatter(object):
             retval += "-> %s" % repr(self.output)
         return retval
 
-    @staticmethod
-    def _format_args(args):
+    @classmethod
+    def _format_args(cls, args):
         items = []
         for arg in args:
-            if isinstance(arg, unicode):
-                arg = safeunicode.get_string(arg)
-
-            if isinstance(arg, (int, str, dict)):
-                items.append(repr(arg))
-            else:
-                items.append(str(arg))
+            items.append(cls._format_value(arg))
 
         return items
 
-    @staticmethod
-    def _format_kargs(kargs):
-        return ['%s=%s' % (key, repr(val))
+    @classmethod
+    def _format_kargs(cls, kargs):
+        return ['%s=%s' % (key, cls._format_value(val))
                 for key, val in sorted(kargs.items())]
 
+    @classmethod
+    def _format_value(cls, arg):
+        if isinstance(arg, unicode):
+            arg = get_string(arg)
+
+        if isinstance(arg, (int, str, dict)):
+            return repr(arg)
+        else:
+            return str(arg)
+
 
 class PropertyGet(Invocation):
     def __repr__(self):
@@ -357,14 +363,18 @@ class Property(object):
 
 class AttributeFactory(object):
     typemap = {
-        'instancemethod': Method,
-        'property':       Property
+        'instancemethod':    Method,
+        'method_descriptor': Method,
+        'property':          Property,
+        # -- python3 --
+        'method':            Method,
+        'function':          Method,
         }
 
     @classmethod
     def create(cls, double, key):
-        typeid = double._proxy.get_attr_typeid(key)
-        return cls.typemap[typeid](double, key)
+        typename = double._proxy.get_attr_typename(key)
+        return cls.typemap[typename](double, key)
 
 
 class SpyBase(object):
